/*--------------------------------------------------------------------------
 *  Copyright 2008 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// utgb-shell Project
//
// GenomeReadFormat.java
// Since: Apr 10, 2009
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-shell/src/main/java/org/utgenome/shell/db/read/GenomeReadFormatDBBuilder.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.shell.db.read;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import org.xerial.core.XerialError;
import org.xerial.core.XerialErrorCode;
import org.xerial.util.log.Logger;

/**
 * Reader
 * 
 * @author leo
 * 
 */
public class GenomeReadFormatDBBuilder {

	private static Logger _logger = Logger.getLogger(GenomeReadFormatDBBuilder.class);

	public static class ReferenceReader {

		private static int count = 1;

		private String name;
		private long start;
		private String strand;
		private int tag;
		private long s2st;
		private long s2ed;
		private String s2strand;
		private int score;

		private final int id;

		private StringBuilder sequenceBuilder = new StringBuilder();

		public ReferenceReader() {
			id = count++;
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public long getStart() {
			return start;
		}

		public void setStart(long start) {
			this.start = start;
		}

		public String getStrand() {
			return strand;
		}

		public void setStrand(String strand) {
			this.strand = strand;
		}

		public int getTag() {
			return tag;
		}

		public void setTag(int tag) {
			this.tag = tag;
		}

		public long getS2st() {
			return s2st;
		}

		public void setS2st(long s2st) {
			this.s2st = s2st;
		}

		public long getS2ed() {
			return s2ed;
		}

		public void setS2ed(long s2ed) {
			this.s2ed = s2ed;
		}

		public String getS2strand() {
			return s2strand;
		}

		public void setS2strand(String s2strand) {
			this.s2strand = s2strand;
		}

		public int getScore() {
			return score;
		}

		public void setScore(int score) {
			this.score = score;
		}

		public void addRead(Read read) throws SQLException {
			// add read data
			if (_logger.isDebugEnabled())
				_logger.debug("add read: " + read);

			stat.execute(String.format("insert into read values(%d, '%s', %d, %d, %d, %d, '%s', '%s')", this.id, read.getName(), read.getView_start(), read
					.getView_end(), read.getStart(), read.getEnd(), read.getStrand(), read.getSequence()));

		}

		public void appendSequence(String seq) {
			sequenceBuilder.append(seq);
		}

		@Override
		public String toString() {
			return String.format("reference: name=%s, start=%d, strand=%s\nsequence=%s", name, start, strand, sequenceBuilder.toString());
		}

	}

	private static Connection conn = null;
	private static Statement stat = null;

	public GenomeReadFormatDBBuilder() {
		// TODO Auto-generated constructor stub
	}

	public GenomeReadFormatDBBuilder(String dbFilePath) throws SQLException {
		try {
			Class.forName("org.sqlite.JDBC");
			conn = DriverManager.getConnection("jdbc:sqlite:" + dbFilePath);

			conn.setAutoCommit(true);
			stat = conn.createStatement();
			stat.execute("pragma synchronous=off");
			conn.setAutoCommit(false);

			// prepare table
			stat.execute("create table coordinate('group' text, species text, revision text)");
			stat.execute("create table reference(id integer primary key, name text, start integer, strand text, tag integer, score integer, sequence text)");
			stat
					.execute("create table read(reference_id integer, name text, view_start integer, view_end integer, start integer, end integer, strand text, sequence text)");
		}
		catch (ClassNotFoundException e) {
			throw new XerialError(XerialErrorCode.INVALID_STATE, "sqlite JDBC not found");
		}

	}

	public void setCoordinate(Coordinate coordinate) throws SQLException {
		if (_logger.isDebugEnabled())
			_logger.debug("set coordinate: " + coordinate);

		stat
				.execute(String.format("insert into coordinate values('%s', '%s', '%s')", coordinate.getGroup(), coordinate.getSpecies(), coordinate
						.getRevision()));
	}

	public void addReference(ReferenceReader reference) throws SQLException {
		if (_logger.isDebugEnabled())
			_logger.debug("add reference: " + reference);

		stat.execute(String.format("insert into reference values(%d, '%s', %d, '%s', %d, %d, '%s')", reference.id, reference.name, reference.start,
				reference.strand, reference.tag, reference.score, reference.sequenceBuilder.toString()));

	}

	public void commit() throws SQLException {
		conn.commit();
		conn.close();
		_logger.info("commit done.");

		stat = null;
		conn = null;
	}

}
